@@ -21,11 +21,18 @@ from abc import abstractmethod
 import asyncio
 from collections.abc import AsyncIterable
 from dataclasses import dataclass
-from ipaddress import ip_address
 import logging
 
 import async_timeout
-from zeroconf import DNSPointer, ServiceListener, ServiceStateChange, Zeroconf
+from zeroconf import (
+    BadTypeInNameException,
+    DNSPointer,
+    IPVersion,
+    ServiceListener,
+    ServiceStateChange,
+    Zeroconf,
+    current_time_millis,
+)
 from zeroconf.asyncio import AsyncServiceBrowser, AsyncServiceInfo, AsyncZeroconf
 
 from aiohomekit.characteristic_cache import CharacteristicCacheType
@@ -51,33 +58,8 @@ _TIMEOUT_MS = 3000
 logger = logging.getLogger(__name__)
 
 
-def _first_non_link_local_address(
-    addresses: list[bytes] | list[str],
-) -> str | None:
-    """Return the first ipv6 or non-link local ipv4 address, preferring IPv4."""
-    for address in addresses:
-        ip_addr = ip_address(address)
-        if (
-            not ip_addr.is_link_local
-            and not ip_addr.is_unspecified
-            and ip_addr.version == 4
-        ):
-            return str(ip_addr)
-    # If we didn't find a good IPv4 address, check for IPv6 addresses.
-    for address in addresses:
-        ip_addr = ip_address(address)
-        if (
-            not ip_addr.is_link_local
-            and not ip_addr.is_unspecified
-            and ip_addr.version == 6
-        ):
-            return str(ip_addr)
-    return None
-
-
 @dataclass
 class HomeKitService:
-
     name: str
     id: str
     model: str
@@ -96,9 +78,26 @@ class HomeKitService:
 
     @classmethod
     def from_service_info(cls, service: AsyncServiceInfo) -> HomeKitService:
-        if not (addresses := service.parsed_addresses()):
+        if not (addresses := service.ip_addresses_by_version(IPVersion.All)):
             raise ValueError("Invalid HomeKit Zeroconf record: Missing address")
 
+        address: str | None = None
+        #
+        # Zeroconf addresses are guaranteed to be returned in LIFO (last in, first out)
+        # order with IPv4 addresses first and IPv6 addresses second.
+        #
+        # This means the first address will always be the most recently added
+        # address of the given IP version.
+        #
+        for ip_addr in addresses:
+            if not ip_addr.is_link_local and not ip_addr.is_unspecified:
+                address = str(ip_addr)
+                break
+        if not address:
+            raise ValueError(
+                "Invalid HomeKit Zeroconf record: Missing non-link-local or unspecified address"
+            )
+
         props: dict[str, str] = {
             k.decode("utf-8").lower(): v.decode("utf-8")
             for (k, v) in service.properties.items()
@@ -118,8 +117,8 @@ class HomeKitService:
             category=Categories(int(props.get("ci", 1))),
             protocol_version=props.get("pv", "1.0"),
             type=service.type,
-            address=_first_non_link_local_address(addresses),
-            addresses=addresses,
+            address=address,
+            addresses=[str(ip_addr) for ip_addr in addresses],
             port=service.port,
         )
 
@@ -149,7 +148,6 @@ def find_brower_for_hap_type(azc: AsyncZeroconf, hap_type: str) -> AsyncServiceB
 
 
 class ZeroconfDiscovery(AbstractDiscovery):
-
     description: HomeKitService
 
     def __init__(self, description: HomeKitService):
@@ -230,13 +228,17 @@ class ZeroconfController(AbstractController):
 
     async def _async_update_from_cache(self, zc: Zeroconf) -> None:
         """Load the records from the cache."""
-        infos = [
-            AsyncServiceInfo(self.hap_type, record.alias)
-            for record in self._async_get_ptr_records(zc)
-        ]
-        tasks = []
-        for info in infos:
-            if info.load_from_cache(self._async_zeroconf_instance.zeroconf):
+        tasks: list[asyncio.Task] = []
+        now = current_time_millis()
+        for record in self._async_get_ptr_records(zc):
+            try:
+                info = AsyncServiceInfo(self.hap_type, record.alias)
+            except BadTypeInNameException as ex:
+                logger.debug(
+                    "Ignoring record with bad type in name: %s: %s", record.alias, ex
+                )
+                continue
+            if info.load_from_cache(zc, now):
                 self._async_handle_loaded_service_info(info)
             else:
                 tasks.append(self._async_handle_service(info))
@@ -254,12 +256,16 @@ class ZeroconfController(AbstractController):
         service_type: str,
         name: str,
         state_change: ServiceStateChange,
-    ):
+    ) -> None:
         if service_type != self.hap_type:
             return
 
         if not (debouncer := self._debouncers.get(name)):
-            info = AsyncServiceInfo(service_type, name)
+            try:
+                info = AsyncServiceInfo(service_type, name)
+            except BadTypeInNameException as ex:
+                logger.debug("Ignoring record with bad type in name: %s: %s", name, ex)
+                return
 
             async def _async_handle_service():
                 if info.load_from_cache(self._async_zeroconf_instance.zeroconf):
@@ -291,7 +297,7 @@ class ZeroconfController(AbstractController):
             return discovery
 
         waiters = self._waiters.setdefault(device_id, [])
-        waiter = asyncio.Future()
+        waiter = asyncio.get_running_loop().create_future()
         waiters.append(waiter)
 
         try:
@@ -340,7 +346,8 @@ class ZeroconfController(AbstractController):
 
         if waiters := self._waiters.pop(description.id, None):
             for waiter in waiters:
-                waiter.set_result(discovery)
+                if not waiter.cancelled():
+                    waiter.set_result(discovery)
 
     @abstractmethod
     def _make_discovery(self, description: HomeKitService) -> AbstractDiscovery:
